# Copyright 2023-2024 Deepgram SDK contributors. All Rights Reserved.
# Use of this source code is governed by a MIT license that can be found in the LICENSE file.
# SPDX-License-Identifier: MIT

from ...utils import verboselogs

# Constants for microphone
LOGGING = verboselogs.WARNING
CHANNELS = 1
RATE = 16000
CHUNK = 8194
